@@ -202,7 +202,7 @@ static int64_t
 file_skip(struct archive *a, void *data, int64_t request)
 {
   struct mydata *mydata = (struct mydata *)data;
-  int64_t result = lseek(mydata->fd, SEEK_CUR, request);
+  int64_t result = lseek(mydata->fd, request, SEEK_CUR);
   if (result >= 0)
     return result;
   archive_set_error(a, errno, "Error seeking in '%s'", mydata->filename);
