@@ -23,7 +23,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: head/lib/libarchive/test/test_pax_filename_encoding.c 201247 2009-12-30 05:59:21Z kientzle $");
 
 #include <locale.h>
 
@@ -335,8 +334,7 @@ test_pax_filename_encoding_3(void)
 /*
  * Verify that KOI8-R filenames are correctly translated to Unicode and UTF-8.
  */
-static void
-test_pax_filename_encoding_KOI8R(void)
+DEFINE_TEST(test_pax_filename_encoding_KOI8R)
 {
   	struct archive *a;
   	struct archive_entry *entry;
@@ -348,7 +346,7 @@ test_pax_filename_encoding_KOI8R(void)
 		return;
 	}
 
-	/* Check if the paltform completely supports the string conversion. */
+	/* Check if the platform completely supports the string conversion. */
 	a = archive_write_new();
 	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
 	if (archive_write_set_options(a, "hdrcharset=UTF-8") != ARCHIVE_OK) {
@@ -382,8 +380,7 @@ test_pax_filename_encoding_KOI8R(void)
 /*
  * Verify that CP1251 filenames are correctly translated to Unicode and UTF-8.
  */
-static void
-test_pax_filename_encoding_CP1251(void)
+DEFINE_TEST(test_pax_filename_encoding_CP1251)
 {
   	struct archive *a;
   	struct archive_entry *entry;
@@ -396,7 +393,7 @@ test_pax_filename_encoding_CP1251(void)
 		return;
 	}
 
-	/* Check if the paltform completely supports the string conversion. */
+	/* Check if the platform completely supports the string conversion. */
 	a = archive_write_new();
 	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
 	if (archive_write_set_options(a, "hdrcharset=UTF-8") != ARCHIVE_OK) {
@@ -430,8 +427,7 @@ test_pax_filename_encoding_CP1251(void)
 /*
  * Verify that EUC-JP filenames are correctly translated to Unicode and UTF-8.
  */
-static void
-test_pax_filename_encoding_EUCJP(void)
+DEFINE_TEST(test_pax_filename_encoding_EUCJP)
 {
   	struct archive *a;
   	struct archive_entry *entry;
@@ -443,7 +439,7 @@ test_pax_filename_encoding_EUCJP(void)
 		return;
 	}
 
-	/* Check if the paltform completely supports the string conversion. */
+	/* Check if the platform completely supports the string conversion. */
 	a = archive_write_new();
 	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
 	if (archive_write_set_options(a, "hdrcharset=UTF-8") != ARCHIVE_OK) {
@@ -478,8 +474,7 @@ test_pax_filename_encoding_EUCJP(void)
 /*
  * Verify that CP932/SJIS filenames are correctly translated to Unicode and UTF-8.
  */
-static void
-test_pax_filename_encoding_CP932(void)
+DEFINE_TEST(test_pax_filename_encoding_CP932)
 {
   	struct archive *a;
   	struct archive_entry *entry;
@@ -492,7 +487,7 @@ test_pax_filename_encoding_CP932(void)
 		return;
 	}
 
-	/* Check if the paltform completely supports the string conversion. */
+	/* Check if the platform completely supports the string conversion. */
 	a = archive_write_new();
 	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
 	if (archive_write_set_options(a, "hdrcharset=UTF-8") != ARCHIVE_OK) {
@@ -528,8 +523,7 @@ test_pax_filename_encoding_CP932(void)
  * Verify that KOI8-R filenames are not translated to Unicode and UTF-8
  * when using hdrcharset=BINARY option.
  */
-static void
-test_pax_filename_encoding_KOI8R_BINARY(void)
+DEFINE_TEST(test_pax_filename_encoding_KOI8R_BINARY)
 {
   	struct archive *a;
   	struct archive_entry *entry;
@@ -568,8 +562,7 @@ test_pax_filename_encoding_KOI8R_BINARY(void)
  * Pax format writer only accepts both BINARY and UTF-8.
  * If other character-set name is specified, you will get ARCHIVE_FAILED.
  */
-static void
-test_pax_filename_encoding_KOI8R_CP1251(void)
+DEFINE_TEST(test_pax_filename_encoding_KOI8R_CP1251)
 {
   	struct archive *a;
 
@@ -586,16 +579,162 @@ test_pax_filename_encoding_KOI8R_CP1251(void)
 	assertEqualInt(ARCHIVE_OK, archive_write_free(a));
 }
 
+/*
+ * Verify that unicode filenames are correctly preserved on Windows
+ */
+DEFINE_TEST(test_pax_filename_encoding_UTF16_win)
+{
+#if !defined(_WIN32) || defined(__CYGWIN__)
+	skipping("This test is meant to verify unicode string handling"
+		" on Windows with UTF-16 names");
+	return;
+#else
+	struct archive *a;
+	struct archive_entry *entry;
+	char buff[0x2000];
+	char *p;
+	size_t used;
+
+	/*
+	 * Don't call setlocale because we're verifying that the '_w' functions
+	 * work as expected when 'hdrcharset' is UTF-8
+	 */
+
+	/* Check if the platform completely supports the string conversion. */
+	a = archive_write_new();
+	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
+	if (archive_write_set_options(a, "hdrcharset=UTF-8") != ARCHIVE_OK) {
+		skipping("This system cannot convert character-set"
+		    " from UTF-16 to UTF-8.");
+		archive_write_free(a);
+		return;
+	}
+	archive_write_free(a);
+
+	/*
+	 * Create a new archive handle with default charset handling
+	 */
+	a = archive_write_new();
+	assertEqualInt(ARCHIVE_OK, archive_write_set_format_pax(a));
+	assertEqualInt(ARCHIVE_OK,
+	    archive_write_open_memory(a, buff, sizeof(buff), &used));
+
+	/* Part 1: file */
+	entry = archive_entry_new2(a);
+	archive_entry_copy_pathname_w(entry, L"\u4f60\u597d.txt");
+	archive_entry_set_filetype(entry, AE_IFREG);
+	archive_entry_set_size(entry, 0);
+	assertEqualInt(ARCHIVE_OK, archive_write_header(a, entry));
+
+	/* Part 2: directory */
+	/* NOTE: Explicitly not adding trailing slash to test that code path */
+	archive_entry_copy_pathname_w(entry, L"\u043f\u0440\u0438");
+	archive_entry_set_filetype(entry, AE_IFDIR);
+	archive_entry_set_size(entry, 0);
+	assertEqualInt(ARCHIVE_OK, archive_write_header(a, entry));
+
+	/* Part 3: symlink */
+	archive_entry_copy_pathname_w(entry, L"\u518d\u89c1.txt");
+	archive_entry_copy_symlink_w(entry, L"\u4f60\u597d.txt");
+	archive_entry_set_filetype(entry, AE_IFLNK);
+	archive_entry_set_symlink_type(entry, AE_SYMLINK_TYPE_FILE);
+	archive_entry_set_size(entry, 0);
+	assertEqualInt(ARCHIVE_OK, archive_write_header(a, entry));
+
+	/* Part 4: hardlink */
+	archive_entry_copy_pathname_w(entry, L"\u665a\u5b89.txt");
+	archive_entry_copy_hardlink_w(entry, L"\u4f60\u597d.txt");
+	archive_entry_set_filetype(entry, AE_IFREG);
+	archive_entry_set_size(entry, 0);
+	assertEqualInt(ARCHIVE_OK, archive_write_header(a, entry));
+
+	archive_entry_free(entry);
+	assertEqualInt(ARCHIVE_OK, archive_write_free(a));
+
+	/*
+	 * Examine the bytes to ensure the filenames ended up UTF-8
+	 * encoded as we expect.
+	 */
+
+	/* Part 1: file */
+	p = buff + 0;
+	assertEqualString(p + 0, "PaxHeader/\xE4\xBD\xA0\xE5\xA5\xBD.txt"); /* File name */
+	assertEqualInt(p[156], 'x'); /* Pax extension header */
+	p += 512; /* Pax extension body */
+	assertEqualString(p + 0, "19 path=\xE4\xBD\xA0\xE5\xA5\xBD.txt\n");
+	p += 512; /* Ustar header */
+	assertEqualString(p + 0, "\xE4\xBD\xA0\xE5\xA5\xBD.txt"); /* File name */
+	assertEqualInt(p[156], '0');
+
+	/* Part 2: directory */
+	p += 512; /* Pax extension header */
+	assertEqualString(p + 0, "PaxHeader/\xD0\xBF\xD1\x80\xD0\xB8"); /* File name */
+	assertEqualInt(p[156], 'x');
+	p += 512; /* Pax extension body */
+	assertEqualString(p + 0, "16 path=\xD0\xBF\xD1\x80\xD0\xB8/\n");
+	p += 512; /* Ustar header */
+	assertEqualString(p + 0, "\xD0\xBF\xD1\x80\xD0\xB8/"); /* File name */
+	assertEqualInt(p[156], '5'); /* directory */
+
+	/* Part 3: symlink */
+	p += 512; /* Pax Extension Header */
+	assertEqualString(p + 0, "PaxHeader/\xE5\x86\x8D\xE8\xA7\x81.txt"); /* File name */
+	p += 512; /* Pax extension body */
+	assertEqualString(p + 0,
+			  "19 path=\xE5\x86\x8D\xE8\xA7\x81.txt\n"
+			  "23 linkpath=\xE4\xBD\xA0\xE5\xA5\xBD.txt\n"
+			  "31 LIBARCHIVE.symlinktype=file\n");
+	p += 512; /* Ustar header */
+	assertEqualString(p + 0, "\xE5\x86\x8D\xE8\xA7\x81.txt"); /* File name */
+	assertEqualInt(p[156], '2'); /* symlink */
+	assertEqualString(p + 157, "\xE4\xBD\xA0\xE5\xA5\xBD.txt"); /* link name */
+
+	/* Part 4: hardlink */
+	p += 512; /* Pax extension header */
+	assertEqualString(p + 0, "PaxHeader/\xE6\x99\x9A\xE5\xAE\x89.txt"); /* File name */
+	p += 512; /* Pax extension body */
+	assertEqualString(p + 0,
+			  "19 path=\xE6\x99\x9A\xE5\xAE\x89.txt\n"
+			  "23 linkpath=\xE4\xBD\xA0\xE5\xA5\xBD.txt\n"
+			  "31 LIBARCHIVE.symlinktype=file\n");
+	p += 512; /* Ustar header */
+	assertEqualString(p + 0, "\xE6\x99\x9A\xE5\xAE\x89.txt"); /* File name */
+	assertEqualInt(p[156], '1'); /* hard link */
+	assertEqualString(p + 157, "\xE4\xBD\xA0\xE5\xA5\xBD.txt"); /* link name */
+
+	/*
+	 * Read back the archive to see if we get the original names
+	 */
+	a = archive_read_new();
+	archive_read_support_format_all(a);
+	archive_read_support_filter_all(a);
+	assertEqualInt(0, archive_read_open_memory(a, buff, used));
+
+	/* Read part 1: file */
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &entry));
+	assertEqualWString(L"\u4f60\u597d.txt", archive_entry_pathname_w(entry));
+
+	/* Read part 2: directory */
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &entry));
+	assertEqualWString(L"\u043f\u0440\u0438/", archive_entry_pathname_w(entry));
+
+	/* Read part 3: symlink */
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &entry));
+	assertEqualWString(L"\u518d\u89c1.txt", archive_entry_pathname_w(entry));
+	assertEqualWString(L"\u4f60\u597d.txt", archive_entry_symlink_w(entry));
+
+	/* Read part 4: hardlink */
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &entry));
+	assertEqualWString(L"\u665a\u5b89.txt", archive_entry_pathname_w(entry));
+	assertEqualWString(L"\u4f60\u597d.txt", archive_entry_hardlink_w(entry));
+
+	archive_free(a);
+#endif
+}
 
 DEFINE_TEST(test_pax_filename_encoding)
 {
 	test_pax_filename_encoding_1();
 	test_pax_filename_encoding_2();
 	test_pax_filename_encoding_3();
-	test_pax_filename_encoding_KOI8R();
-	test_pax_filename_encoding_CP1251();
-	test_pax_filename_encoding_EUCJP();
-	test_pax_filename_encoding_CP932();
-	test_pax_filename_encoding_KOI8R_BINARY();
-	test_pax_filename_encoding_KOI8R_CP1251();
 }
