@@ -23,7 +23,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: head/lib/libarchive/test/test_read_pax_truncated.c 189483 2009-03-07 03:34:34Z kientzle $");
 
 DEFINE_TEST(test_read_pax_truncated)
 {
@@ -48,8 +47,8 @@ DEFINE_TEST(test_read_pax_truncated)
 	assert((ae = archive_entry_new()) != NULL);
 	archive_entry_copy_pathname(ae, "file");
 	archive_entry_set_mode(ae, S_IFREG | 0755);
-	for (i = 0; i < filedata_size; i++)
-		filedata[i] = (unsigned char)rand();
+	fill_with_pseudorandom_data(filedata, filedata_size);
+
 	archive_entry_set_atime(ae, 1, 2);
 	archive_entry_set_ctime(ae, 3, 4);
 	archive_entry_set_mtime(ae, 5, 6);
@@ -82,7 +81,7 @@ DEFINE_TEST(test_read_pax_truncated)
 			assertEqualIntA(a, ARCHIVE_FATAL, archive_read_next_header(a, &ae));
 			goto wrap_up;
 		} else {
-			failure("Archive truncated to %d bytes", i);
+			failure("Archive truncated to %zu bytes", i);
 			assertEqualIntA(a, 0, archive_read_next_header(a, &ae));
 		}
 
@@ -91,7 +90,7 @@ DEFINE_TEST(test_read_pax_truncated)
 			assertEqualIntA(a, ARCHIVE_FATAL, archive_read_data(a, filedata, filedata_size));
 			goto wrap_up;
 		} else {
-			failure("Archive truncated to %d bytes", i);
+			failure("Archive truncated to %zu bytes", i);
 			assertEqualIntA(a, filedata_size,
 			    archive_read_data(a, filedata, filedata_size));
 		}
@@ -103,7 +102,7 @@ DEFINE_TEST(test_read_pax_truncated)
 		 * does not return an error if it can't consume
 		 * it.) */
 		if (i < 1536 + 512*((filedata_size + 511)/512) + 512) {
-			failure("i=%d minsize=%d", i,
+			failure("i=%zu minsize=%zu", i,
 			    1536 + 512*((filedata_size + 511)/512) + 512);
 			assertEqualIntA(a, ARCHIVE_FATAL,
 			    archive_read_next_header(a, &ae));
